<?php

// CRON[5498]: (pam_unix) session closed for user root
// passwd[9618]: (pam_unix) could not recover authentication token
// passwd[9618]: (pam_unix) authentication failure; logname=x uid=1000 euid=0 tty= ruser= rhost=  user=x
// gdm[3334]: (pam_unix) authentication failure; logname= uid=0 euid=0 tty=:0 ruser= rhost=  user=x
// gdm[3334]: (pam_unix) session opened for user x by (uid=0)
// gdm[3334]: (pam_unix) check pass; user unknown
$messages[] = array('|^\(pam_unix\) |U', 'pam');

function callback_pam($data) {

 // not interactive/scheduled
 if (
  preg_match('|^\(pam_unix\) session opened for user .* by \(uid=.*\)$|U', $data['message']) && 
  preg_match('|^CRON\[\d+\]$|U', $data['source'])
 ) return array(
   'level' => LEVEL_NOTICE,
   'name' => 'Cron has opened a PAM session!',
   'description' => 'A full dump follows: '.print_r($data, TRUE)
  );

 // interactive logins
 if (
  preg_match('|^\(pam_unix\) session opened for user .* by \(uid=.*\)$|U', $data['message']) && 
  (
   preg_match('|^su\[\d+\]$|U', $data['source']) ||
   preg_match('|^gdm\[\d+\]$|U', $data['source'])
  )
 ) return array(
   'level' => LEVEL_WARNING,
   'name' => 'Cron has opened a PAM session!',
   'description' => 'A full dump follows: '.print_r($data, TRUE)
  );

 if (
  preg_match('|^\(pam_unix\) session closed for user .*$|U', $data['message']) && 
  (
   preg_match('|^CRON\[\d+\]$|U', $data['source']) || 
   preg_match('|^su\[\d+\]$|U', $data['source']) ||
   preg_match('|^gdm\[\d+\]$|U', $data['source']) // dunno if this exists. todo check
  )
 ) return array(
   'level' => LEVEL_NOTICE,
   'name' => 'Cron has closed a PAM session!',
   'description' => 'A full dump follows: '.print_r($data, TRUE)
  );

 if (
  preg_match('|^\(pam_unix\) could not recover authentication token$|U', $data['message']) && 
  preg_match('|^passwd\[\d+\]$|U', $data['source'])
 ) return array(
   'level' => LEVEL_WARNING,
   'name' => 'PAM failed recovering an auth token!',
   'description' => 'This often means a failed login. A full dump follows: '.print_r($data, TRUE)
  );

 if (
  preg_match('|^\(pam_unix\) authentication failure; .*$|U', $data['message']) && 
  (
   preg_match('|^passwd\[\d+\]$|U', $data['source']) ||
   preg_match('|^gdm\[\d+\]$|U', $data['source'])
  )
 ) return array(
   'level' => LEVEL_CRITICAL,
   'name' => 'An authentication failed against PAM!',
   'description' => 'This means a failed login. A full dump follows: '.print_r($data, TRUE)
  );

 if (
  preg_match('|^\(pam_unix\) check pass; user unknown$|U', $data['message']) && 
  preg_match('|^gdm\[\d+\]$|U', $data['source'])
 ) return array(
   'level' => LEVEL_CRITICAL,
   'name' => 'User unknown PAM!',
   'description' => 'This means a failed login. A full dump follows: '.print_r($data, TRUE)
  );

 return array(
  'level' => LEVEL_CRITICAL,
  'name' => 'Anomaly detected in PAM judge!',
  'description' => 'A full dump follows: '.print_r($data, TRUE)
 );
}

?>
